{% macro form_errors(form, hiddens=True) %}
  {%- if form.errors %}
    {%- for fieldname, errors in form.errors.items() %}
      {%- if bootstrap_is_hidden_field(form[fieldname]) and hiddens or
             not bootstrap_is_hidden_field(form[fieldname]) and hiddens != 'only' %}
        {%- for error in errors %}
          <p class="error">{{error}}</p>
        {%- endfor %}
      {%- endif %}
    {%- endfor %}
  {%- endif %}
{%- endmacro %}

{% macro quick_form(form) %}
  {{ form | render_form(**kwargs) }}
{%- endmacro %}

{% macro form_field(field) %}
  {{ field | render_field(**kwargs) }}
{% endmacro %}
